@@ -1,7 +1,7 @@
 /// \file
 // Range v3 library
 //
-//  Copyright Eric Niebler 2013-2014
+//  Copyright Eric Niebler 2013-present
 //
 //  Use, modification and distribution is subject to the
 //  Boost Software License, Version 1.0. (See accompanying
@@ -40,7 +40,6 @@
 /// Concept-checking classes and utilities
 
 RANGES_DIAGNOSTIC_PUSH
-RANGES_DIAGNOSTIC_IGNORE_PRAGMAS
 RANGES_DIAGNOSTIC_IGNORE_CXX17_COMPAT
 
 namespace ranges
@@ -103,8 +102,9 @@ namespace ranges
         template<typename ...Ts>
         using common_reference_t = meta::_t<common_reference<Ts...>>;
 
-        template<typename, typename = void>
+        template<typename>
         struct result_of;
+
         template<typename Sig>
         using result_of_t = meta::_t<result_of<Sig>>;
 
@@ -393,12 +393,11 @@ namespace ranges
         struct ordered_less;
         struct ident;
 
-        enum cardinality
+        enum cardinality : std::ptrdiff_t
         {
             infinite = -3,
             unknown = -2,
-            finite = -1,
-            _max_ = INT_MAX
+            finite = -1
         };
 
         template<typename Rng, typename Void = void>
@@ -482,15 +481,12 @@ namespace ranges
         template<typename I, typename S = I>
         struct sized_iterator_range;
 
-        template<typename T, bool RValue = false>
+        template<typename T>
         struct reference_wrapper;
 
         template<typename>
         struct is_reference_wrapper;
 
-        template<typename T>
-        using rvalue_reference_wrapper = reference_wrapper<T, true>;
-
         // Views
         //
         template<typename Rng, typename Pred>
